#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de los resumenes de proyecto. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para presentar un resumen de proyecto según diversos parámetros

class ResumenProyectoController < ApplicationController

  before_filter :criterios_gasto, :only => [:gasto]
  before_filter :criterios_presupuesto, :only => [:presupuesto]

  def criterios_gasto
    if params[:financiador_implementador] == "" 
      flash[:error] = _("Debe seleccionar agente. Para ello primero seleccione tipo implementador o financiador.")
      redirect_to :action => "gasto", :sin_layout =>  (params[:sin_layout] ? true : false)
    end
  end

  def criterios_presupuesto
    if params[:selector] and (params[:selector][:etapa].nil? or params[:selector][:etapa]=="" )
      flash[:error] = _("Debe seleccionar etapa. Si no existe ninguna debe dar de alta para este proyecto.")
      redirect_to :action => "presupuesto", :sin_layout =>  (params[:sin_layout] ? true : false)
    end
  end



  def index
    redirect_to :action => 'presupuesto'
  end
  
        # en proyectos: selecciona la moneda y llama a estado_presupuesto si ya hay moneda y actividad/partida
  def presupuesto
    @listado_moneda = @proyecto.moneda.collect{|e| [e.nombre, e.id]} + [["Todas (con tasa aplicada)", "todas"]]
    @listado_etapa = @proyecto.etapa.collect{|e| [e.nombre, e.id]}

    # Hacemos la busqueda si se seleccionan los criterios
    if params[:selector] and params[:selector][:moneda] != '' and params[:selector][:partida_actividad] != '' and params[:selector][:etapa] != ''
      estado_presupuesto 
      respond_to do |format|
        format.html do 
          render :action => 'presupuesto' , :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == "0"
        format.xls do
          @nombre = "Resumen Presupuesto"
          render 'comunes_xls/resumen_tabla', :layout => false
        end if params[:selector][:fichero] == "1"
      end
    else
      render :action => 'presupuesto' , :layout => (params[:sin_layout] ? false : true)
    end
  end

  def estado_presupuesto
    partidas = Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    actividades = @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    implementadores = @proyecto.implementador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    financiadores = @proyecto.financiador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    partidas_proyecto = @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    
    case  [params[:selector][:partida_actividad],  params[:selector][:financiador_implementador]]
      when ["partida", "implementador"] then
         datos = VPresupuesto.partida_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = partidas, implementadores
      when ["partida", "financiador"] then
         datos = VPresupuesto.partida_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = partidas, financiadores
      when ["actividad", "implementador"] then
         datos = VPresupuesto.actividad_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = actividades, implementadores
      when ["actividad", "financiador"] then
         datos = VPresupuesto.actividad_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = actividades, financiadores
      when ["partida_proyecto", "implementador"] then
         datos = VPresupuesto.partida_proyecto_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = partidas_proyecto, implementadores
      when ["partida_proyecto", "financiador"] then
         datos = VPresupuesto.partida_proyecto_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = partidas_proyecto, financiadores
    end       

    titulo =[ "Resumen de presupuesto.",
               "Moneda: #{ params[:selector][:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:selector][:moneda]).abreviatura} " ,
               "Tasa cambio: #{ (params[:selector][:tasa_cambio] == '1' or params[:selector][:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Etapa: #{ Etapa.find(params[:selector][:etapa]).nombre } ",
            	 "Filas: #{ params[:selector][:partida_actividad] }",
		           "Columnas: #{ params[:selector][:financiador_implementador]}" ]

    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => { :columna_suma => true, :fila_suma => true } }]



  end

  # en proyectos: selecciona la moneda, los agentes
  # y llama a busqueda_presupuesto
  def gasto
    @listado_moneda = @proyecto.moneda.collect{|e| [e.nombre, e.id]} + [["Todas las monedas (con tasa aplicada)", "todas"]]
    @listado_agente = @proyecto.implementador.collect{|e| [e.nombre, e.id]} + [["Todos los agentes", "todos"]]
    # Hacemos la busqueda si se seleccionan los criterios
    unless params[:moneda].nil? or params[:partida_actividad].nil? or params[:financiador_implementador].nil? 
      estado_gasto
      respond_to do |format|
        format.html do 
          render :action => "gasto", :layout => (params[:sin_layout] ? false : true)
        end if params[:fichero] == '0'
        format.xls do
          @nombre = "Seguimiento Gasto"
          render 'comunes_xls/resumen_tabla', :layout => false
        end if params[:fichero] == "1"
      end
    else
      render :action => "gasto", :layout => (params[:sin_layout] ? false : true)
    end
  end

  def agentes
    render :partial => "agentes"
  end


  def estado_gasto
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = fecha params["fecha_fin"]
    filas = case params[:partida_actividad]
              when "partida" then Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
              when "actividad" then @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
              when "partida_proyecto" then @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
             end
    columnas = [{"id" => "1", "nombre" => "Presupuesto"}, {"id" => "2", "nombre" => "Gastos"}]

    case  [params[:partida_actividad],  params[:financiador_implementador]]
      when ["partida", "implementador"] then
         presupuestos = VPresupuestoDetallado.partida_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["partida", "financiador"] then
         presupuestos = VPresupuestoDetallado.partida_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["actividad", "implementador"] then
         presupuestos = VPresupuestoDetallado.actividad_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.actividad_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}


      when ["actividad", "financiador"] then
         presupuestos = VPresupuestoDetallado.actividad_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.actividad_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}


     when ["partida_proyecto", "implementador"] then      
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_proyecto_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["partida_proyecto", "financiador"] then
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_proyecto_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

    end 
    datos = presupuestos + gastos 
    titulo =[ "Resumen de presupuesto.",
               "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
               "Tasa cambio: #{ (params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Fechas: #{@fecha_de_inicio.to_s + '/' +  @fecha_de_fin.to_s} ",
            	 "Filas: #{ params[:partida_actividad] }",
		           "Agente: #{ params[:agente] == "todos" ? "Todos los agentes" : Agente.find(params[:agente]).nombre }",
               "Rol del agente: #{ params[:financiador_implementador]}"  ]
    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => {  :columna_resta => true, :fila_suma => true, :desviacion => true } }]
    
  end


	# en proyectos: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = [["Todas", "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    @listado_agentes = @proyecto.implementador.collect{|e| [e.nombre, e.id]}
    @listado_monedas = @proyecto.moneda.collect{|e| [e.nombre, e.id]}
    # Si hemos especificado un libro para ver el informe
    #if params[:selector] and params[:selector][:libro] and params[:selector][:libro] != ''
    if params[:selector] && (params[:selector][:tipo] == "agente" || params[:selector][:tipo] == "cuenta")
      condiciones = { }
      @libro = Libro.find_by_id(params[:selector][:libro]) if params[:selector][:tipo] == "cuenta"
      @agente = @proyecto.implementador.find_by_id(params[:selector][:agente]) if params[:selector][:tipo] == "agente"
      @moneda = Moneda.find_by_id(params[:selector][:moneda]) if params[:selector][:tipo] == "agente"
      @moneda = @libro.moneda if @libro 
      if params[:selector][:etapa] != 'todas'
        @etapa = Etapa.find_by_id(params[:selector][:etapa])
        condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin
      end
      # Condiciones comunes para gastos y movimientos
      condiciones[:moneda_id] = @moneda.id

      # Obtiene todos los gastos de la cuenta en el periodo
      gastos = @proyecto.gasto.find(:all, :conditions => condiciones)

      # Obtiene todas las transferencias de la cuenta o el agente en el periodo
      condiciones[:libro_id] = @libro.id if @libro
      condiciones["libro.agente_id"] = @agente.id if @agente

      transferencias = @proyecto.transferencia.all(:conditions => condiciones, :include => "libro")

      # Genera el array de gastos y transferencias para presentar
      @lineas = Array.new
      @filas_desglosado = Array.new
      total = Hash.new
      entrante=0
      saliente=0
      # Recorre y toma todos los datos de las transferencias
      transferencias.each do |transferencia|
        total[transferencia.tipo.capitalize] = Hash.new if total[transferencia.tipo.capitalize].nil?
        total[transferencia.tipo.capitalize][transferencia.entrante_saliente.capitalize] ||= 0
        total[transferencia.tipo.capitalize][transferencia.entrante_saliente.capitalize] += transferencia.importe
        entrante += transferencia.importe if transferencia.entrante_saliente=="entrante"
        saliente += transferencia.importe if transferencia.entrante_saliente!="entrante"
        @filas_desglosado.push( :contenido => [	transferencia.fecha, transferencia.tipo.capitalize, transferencia.observaciones ? transferencia.observaciones : '',
						transferencia.entrante_saliente=="entrante" ? transferencia.importe : '',
						transferencia.entrante_saliente!="entrante" ? transferencia.importe : '' ])
      end
      
      # Recorre y toma todos los datos de los pagos
      gastos.each do |gasto|
        gasto.pago.each do |pago|
          if (@agente && pago.libro && pago.libro.agente_id == @agente.id) || (@libro && pago.libro_id == @libro.id)
            total["Pago"] = Hash.new if total["Pago"].nil?
            total["Pago"]["Saliente"] = 0 if total["Pago"]["Saliente"].nil?
            total["Pago"]["Saliente"] += pago.importe
            saliente += pago.importe
            @filas_desglosado.push( :contenido => [	pago.fecha, _("Pago"), (pago.observaciones||gasto.concepto),
							'', pago.importe ] )
          end
        end
      end
      # Ordena por fecha
      @filas_desglosado.sort! { |x,y| x[:contenido][0] <=> y[:contenido][0] }

      # Prepara la salida de datos
      @lineas.push( :cabecera => [ [_("Fecha"),"1_2"], [_("Tipo"),"1_2"], [_("Concepto"),"3_2"], [_("Entradas"),"1_2_td"], [_("Salidas"),"1_2_td"] ])
      @lineas += @filas_desglosado
      @lineas.push( :contenido => [ '','','','','' ] )
      @lineas.push( :cabecera => [ [_("Totales"),"1_2"], ["","1_2"], ["","3_2"], [entrante,"1_2_td"], [saliente,"1_2_td"], [entrante - saliente,"1_2_td"] ])
      total.each do |k,v|
        @lineas.push( :contenido => [ '',k,'',v["Entrante"]||'',v["Saliente"]||'','' ] )
      end

      respond_to do |format|
        format.html do
          render :template => 'comunes/arqueo_caja', :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == '0'
        format.xls do
          @nombre = "arqueo_caja"
          @titulo = _("Arqueo ") + (@libro ? _("Cuenta/Caja ") + @libro.nombre : _("Cuentas/Cajas de ") + @agente.nombre ) + " (" + @moneda.nombre + ") / Etapa: " +
			(@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
          render 'comunes_xls/resumen_listado', :layout => false
        end if params[:selector][:fichero] == '1'
      end
    else
      render :template => "comunes/arqueo_caja", :layout => (params[:sin_layout] ? false : true)
    end
  end

	# En proyectos, vista resumen de la matriz por etapa y pais
  def matriz
    @listado_etapa = [["Todas", "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    @listado_pais =  [["Todos", "todos"]] + @proyecto.pais.collect{ |p| [p.nombre, p.id] }.sort{ |a, b| a[0] <=> b[0]}
    if params[:selector]
      @pais = Pais.find(params[:selector][:pais]) if params[:selector][:pais] != "todos"
      @etapa = Etapa.find(params[:selector][:etapa]) if params[:selector][:etapa] != "todas"
      @columnas_nombre = [_("descripcion"), _("indicadores"), _("fuentes de verificacion"), _("hipótesis")]
      @lineas = Array.new
      condiciones = { "proyecto_id" =>  @proyecto.id}
      condiciones["actividad_x_pais.pais_id"] = params[:selector][:pais] if params[:selector][:pais] != "todos"
      condiciones["actividad_x_etapa.etapa_id"] = params[:selector][:etapa] if params[:selector][:etapa] != "todas"
      filas_objetivos = Array.new
      filas_resultados = Array.new
      filas_actividades = Array.new 
      @proyecto.objetivo_especifico.all.each do |objetivo|
        existen_actividades = false
        objetivo.resultado.all.each do |resultado|
          existen_actividades = false
          actividades = resultado.actividad.find(:all, :include => [:actividad_x_pais, :actividad_x_etapa], :conditions => condiciones )
          actividades.each do |actividad|
            nuevas_actividades = desglosa_actividades(actividad,@etapa)
            filas_actividades += nuevas_actividades 
            existen_actividades = true if nuevas_actividades.length > 0 
          end
          filas_resultados += desglosa_fuentes(resultado) if existen_actividades
        end
        filas_objetivos += desglosa_fuentes(objetivo) if existen_actividades 
      end
      # Va rellenando las lineas
      @lineas.push(:cabecera => [ [_("Descripcion"),"3_2"], [_("Indicadores"),"1"], [_("Fuentes de verificacion"),"1"], [_("Hipótesis"),"1"] ]) 
      @lineas += filas_objetivos
      @lineas.push(:cabecera => [ [_("Resultados"),"3_2"], ["","1"], ["","1"], ["","1" ] ])
      @lineas += filas_resultados 
      @lineas.push(:cabecera => [ [_("Actividad"),"3_2"], [_("Recursos"),"1_td"], [_("Costes"),"1_td"] ])
      @lineas += filas_actividades

      respond_to do |format|
        format.html do
          render :action => "matriz", :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == '0'
        format.xls do
          @nombre = "matriz"
          @titulo = "Matriz " + @proyecto.nombre + " / Pais: " + (@pais ? @pais.nombre : "Todos") + " / Etapa: " +
			(@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
          render 'comunes_xls/resumen_listado', :layout => false
        end if params[:selector][:fichero] == '1'
      end
    else
      render :action => "matriz", :layout => (params[:sin_layout] ? false : true)
    end

  end

private


	# Para un objetivo especifico o un resultado, saca hipotesis, indicadores y fuentes de verificacion
  def desglosa_fuentes(objeto)
    filas = Array.new
    hipotesis = objeto.hipotesis
    indicadores = objeto.indicador
    objeto_repetido = nil
    # Si no hay indicadores al menos muestra el codigo del objeto
    filas.push( :contenido => [ objeto.codigo + ": " + objeto.descripcion, "", "", "" ] ) if indicadores.length == 0
    # Si hay indicadores se los recorre para rellenar las filas
    indicadores.each do |indicador|
      indicador_repetido = nil
      fuentes = indicador.fuente_verificacion
      # Si no hay fuentes de verificacion al menos pone el indicador
      filas.push( :contenido => [	objeto_repetido ? "" : objeto.codigo + ": " + objeto.descripcion,
					indicador.codigo + ": " + indicador.descripcion,
					"",
					(hipotesis.length>0 ? hipotesis.shift.descripcion : "") ]) if fuentes.length == 0
      # Si hay fuentes de verificacion las recorre para rellenar las filas
      fuentes.each do |fuente|
        filas.push( :contenido => [	objeto_repetido ? "" : objeto.codigo + ": " + objeto.descripcion,
					indicador_repetido ? "" : indicador.codigo + ": " + indicador.descripcion,
					fuente.codigo + ": " + fuente.descripcion,
					(hipotesis.length>0 ? hipotesis.shift.descripcion : "") ])
        objeto_repetido = true
        indicador_repetido = true
      end
      objeto_repetido = true
    end
    # Si queda alguna hipotesis la mete
    hipotesis.each do |hip|
      filas.push( :contenido => [	"", "", "", hip.descripcion ]) 
    end
    return filas
  end

	# Para una actividad, saca partidas de financiador e importes totales
  def desglosa_actividades(objeto,etapa)
    hash = Hash.new
    filas = Array.new
    importe_total = 0
    # Se recorre todos los presupuestos por actividad
    objeto.presupuesto_x_actividad.each do |pxa|
      # Y en el caso de que el presupuesto sea de la etapa relacionada y tenga asignada una partida lo mete
      if etapa.nil? || (pxa.presupuesto && pxa.presupuesto.etapa_id == etapa.id)
        partida_asociada=pxa.presupuesto.partida.partida_asociada(@proyecto.id) if pxa.presupuesto && pxa.presupuesto.partida
        if partida_asociada
          # aplica la tasa de cambio
          importe = pxa.importe * pxa.presupuesto.tasa_cambio
          hash[partida_asociada.nombre] += importe if hash[partida_asociada.nombre]
          hash[partida_asociada.nombre] = importe unless hash[partida_asociada.nombre]
          importe_total += importe
        end
      end
    end
    # Si no hay presupuesto para la actividad, mete al menos la actividad
    filas.push( :contenido => [		objeto.codigo + ": " + objeto.descripcion, "", ""]) if hash.length == 0
    # Si hay datos en el hash anterior, va rellenando las lineas
    if importe_total != 0
      filas.push( :contenido => [	objeto.codigo + ": " + objeto.descripcion, "", importe_total])
    end
    hash.each do |k,v|
      filas.push( :contenido => [	"", k, v ])
    end
    return filas
  end

end
#done

