#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# ActiveResource que devuelve proyecto desde Alfresco.
class Proyecto < ActiveRecord::Base
  has_many :proyecto_x_pais, :dependent => :destroy
  has_many :pais, :through => :proyecto_x_pais
  belongs_to :agente
  belongs_to :moneda_principal, :class_name => "Moneda", :foreign_key => "moneda_id"
  belongs_to :moneda_intermedia, :class_name => "Moneda", :foreign_key => "moneda_intermedia_id"
  has_one :datos_proyecto
  has_one :espacio
  has_many :usuario_x_proyecto, :dependent => :destroy
  has_many :gasto_x_proyecto, :dependent => :destroy
  has_many :gasto, :through => :gasto_x_proyecto
  has_many :usuario, :through => :usuario_x_proyecto
  has_many :etapa, :dependent => :destroy
  has_many :libro_x_proyecto
  has_many :libro, :through => :libro_x_proyecto
  has_many :transferencia
  has_many :presupuesto
  has_many :definicion_dato, :through => :proyecto_x_definicion_dato
  has_many :proyecto_x_definicion_dato, :dependent => :destroy
  has_many :dato_texto
  has_many :documento, :through => :espacio
  has_many :objetivo_especifico
  has_many :resultado
  has_many :actividad
  has_many :estado, :dependent => :destroy
  has_one  :definicion_estado, :through => :estado, :conditions => ["estado.estado_actual"]
  has_many :tarea
  has_many :partida_financiacion
  has_many :proyecto_x_moneda, :dependent => :destroy
  has_many :moneda, :through => :proyecto_x_moneda, :uniq => true
  has_many :tasa_cambio_proyecto, :dependent => :destroy
  has_and_belongs_to_many :sector_poblacion,  :join_table => "proyecto_x_sector_poblacion"
  has_and_belongs_to_many :area_actuacion,  :join_table => "proyecto_x_area_actuacion"
  has_and_belongs_to_many :sector_intervencion,  :join_table => "proyecto_x_sector_intervencion"
  has_and_belongs_to_many :financiador, :join_table => "proyecto_x_financiador", :class_name => "Agente"
  has_and_belongs_to_many :implementador, :join_table => "proyecto_x_implementador", :class_name => "Agente"
  

  # Para cofinanciacion de proyectos
  has_many :proyecto_x_proyecto, :dependent => :destroy
  has_many :proyecto_x_proyecto_cofinanciado, :foreign_key => 'proyecto_cofinanciador_id', :class_name => 'ProyectoXProyecto', :dependent => :destroy
  has_many :proyecto_cofinanciador, :through => :proyecto_x_proyecto, :class_name => 'Proyecto'
  has_many :proyecto_cofinanciado, :through => :proyecto_x_proyecto_cofinanciado, :class_name => 'Proyecto'

  validate :nombre_mayusculas 
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  validates_uniqueness_of :titulo, :message => _("Título") +   _(" repetido.")
  validates_presence_of :titulo, :message => _("Título") + _(" no puede estar vacío.")
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :agente_id, :message => _("Financiador") + _(" no puede estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no puede estar vacía.")
  
  after_create :crear_espacio
  after_save :nombre_espacio, :cambiar_tasas_cambio
  before_destroy :verificar_borrado

  def crear_espacio
    Espacio.create :nombre => nombre, :proyecto_id => id, :espacio_padre_id => Espacio.find_by_nombre("Proyectos").id , :descripcion => ("Espacio raiz del proyecto: " + nombre)
  end

  def nombre_espacio
    espacio.update_attribute "nombre", nombre
  end

  def dato_dinamico id , tipo 
    eval( "Dato" + tipo ).find(id) unless id == 0
    eval( "Dato" + tipo ).new if id == 0 
  end

  def datos_dinamicos pestana
    # ( dato_integro.select{|d| d.definicion_dato.pestana == pestana } + dato_cadena.select{|d| d.definicion_dato.pestana == pestana }  + dato_texto.select{|d| d.definicion_dato.pestana == pestana } ).sort{ |a,b| a.definicion_dato.rango <=> b.definicion_dato.rango }
    ( dato_texto.select{|d| d.definicion_dato.pestana == pestana } ).sort{ |a,b| a.definicion_dato.rango <=> b.definicion_dato.rango }
  end

  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  # Devuelve los agentes financiadores del proyecto
  def agente_financiador
    return Agente.find(:all, :conditions => {:financiador => true})
  end

  # Devuelve los agentes implementadores del proyecto
  def agente_implementador
    #return Agente.find(:all, :from => ("#{ALFRESCO_SEARCH_URI}proyecto/"+self.id.to_s+"/agente_implementador"))
  end

  def estado_actual
    (estado.select { |a|  a.estado_actual }).first
  end

 # Devuelve el presupuesto total del proyecto en la moneda base o el de la moneda que se especifique
 def presupuesto_total mon=nil, financ=nil
   total = 0.0
   financ = nil unless financ.class.name == "Agente"
   condiciones = {} 
   #fecha_inicio = self.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.fecha_inicio if fecha_fin
   #condiciones = { :fecha => fecha_inicio..fecha_fin } if fecha_fin && fecha_fin.class.name == "Date"
   condiciones["presupuesto_x_agente.agente_id"] = financ.id if financ
   condiciones["presupuesto.moneda_id"] = mon.id if mon and mon.class.name=="Moneda"
   self.presupuesto.all(:include => "presupuesto_x_agente", :conditions => condiciones).each do |pre|
     importe = financ ? pre.presupuesto_x_agente.first.importe : pre.importe
     total += importe if mon
     total += (importe * pre.tasa_cambio).round(2) unless mon
   end
   return total
 end

 # Devuelve el gasto total del proyecto en la moneda base
 def gasto_total financ=nil
   total = 0.0
   financ = nil unless financ.class.name == "Agente"
   condiciones = {}
   condiciones["gasto_x_agente.agente_id"] = financ.id if financ
   self.gasto.all(:include => ["gasto_x_agente"], :conditions => condiciones).each do |gas|
     gxa = gas.gasto_x_agente.first
     gxp = gas.gasto_x_proyecto.find_by_proyecto_id self.id
     importe = financ ? gxa.importe : gxp.importe
     total += (importe * gxp.tasa_cambio).round(2) if importe && gxp
   end
   return total
 end

 # Devuelve el total de transferencias realizadas (para una o todas las monedas)
 def transferido_total mon=nil
   total = 0.0
   if mon && mon.class.name == "Moneda"
     condiciones = ['transferencia.moneda_id = ? AND tipo = "subvencion"', mon.id] if self.moneda_principal == mon
     condiciones = ['transferencia.moneda_id = ? AND libro_receptor_emisor_id AND libro.moneda_id != transferencia.moneda_id', mon.id] unless self.moneda_principal == mon
     self.transferencia.all(:include => "libro_segundo", :conditions => condiciones).each do |trans|
       total += trans.importe if trans.entrante_saliente == "entrante"
       total -= trans.importe unless trans.entrante_saliente == "entrante"
     end
   end
   return total
 end

  private

  def cambiar_tasas_cambio
    # Para modificaciones de proyecto que incluyan moneda principal, actualiza las tasas de cambio
    self.etapa.each { |e| TasaCambio.cambia_moneda(e) } if id_was && moneda_id_changed?
  end

  def verificar_borrado
    if self.datos_proyecto
      (DatosProyecto.column_names - ["id", "proyecto_id"]).each do |v|
        if not self.datos_proyecto.send( v.to_sym ).nil?
          errors.add( "datos_proyecto","hay datos de proyecto " + v.humanize )
        end
      end
    end
    errors.add( "gasto", "hay gastos" ) unless self.gasto_x_proyecto.empty?
    errors.add( "presupuesto", "hay presupuestos" ) unless self.presupuesto.empty?
    errors.add( "objetivo_especifico", "hay objetivos especificos" ) unless self.objetivo_especifico.empty?
    errors.add( "documento", "hay documentos" ) unless self.documento.empty?
    errors.add( "tarea", "hay tareas" ) unless self.tarea.empty?
    # la relacion habtm no tiene :dependent por eso tenemos que borrar las relaciones de financiador y implementador 
    errors.add( _("esta relacionado con agentes como implementadores") ) unless self.implementador == []
    errors.add( _("esta relacionado con agentes como financiadores") ) unless self.financiador == []
    errors.add_to_base( "Un proyecto tiene que estar vacio para poder ser borrado." ) unless errors.empty?
   return false unless errors.empty?
 end

end
